/*
 *   ---- Delphes ----
 *     A Fast Simulator for general purpose LHC detector
 *       S. Ovyn ~~~~ severine.ovyn@uclouvain.be
 *
 *         Center for Particle Physics and Phenomenology (CP3)
 *           Universite Catholique de Louvain (UCL)
 *             Louvain-la-Neuve, Belgium
 *             */

// \brief Trigger class, and some generic definitions

#include "interface/TriggerUtil.h"
#include "TRandom.h"

#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <cstring>

using namespace std;

TriggerTable::TriggerTable() {
  has_been_evaluated=false;
}

void TriggerTable::TriggerCardReader(const string filename){
   ifstream trigger_file(filename.c_str());
   if( !trigger_file.good() ) {
	cout << "ERROR: Trigger input file "<< filename << " not found. Exiting.\n";
	return;
   }

   TriggerBit mybit;
   string temp_string, temp_string_no_comment, trigger_name, trigger_algorithm;
   while ( getline(trigger_file,temp_string) ) {
	string temp_string_no_comment = temp_string.substr(0,temp_string.find("#")); //remove comments
	if(temp_string_no_comment.size()<2) continue;  // avoid empty lines
 	istringstream temp_stream(temp_string_no_comment);

	temp_stream >> trigger_name;
	trigger_algorithm = temp_string_no_comment.substr( temp_string_no_comment.find(">>")+2) ;

        mybit.TriggerBit::GetTrigCondition(trigger_algorithm);
        list_of_trigger_bits.push_back(mybit);
   }
}


void TriggerTable::PrintTriggerTable(string LogName) {

  ofstream f_out(LogName.c_str(),ios::app);
  f_out<<"*                                                                    *"<<"\n";
  f_out<<"#***********************************************                     *"<<"\n";
  f_out<<"# Trigger conditions defined in the trigger card                     *"<<"\n";
  f_out<<"#***********************************************                     *"<<"\n";
  f_out<<"*                                                                    *"<<"\n";

  f_out.close();
 
  for(unsigned int i=0; i < list_of_trigger_bits.size(); i++) {
  list_of_trigger_bits[i].PrintTrigCondition(LogName,i+1);
 }
}

//*************************************************************************

TriggerBit::TriggerBit() {
 has_been_evaluated=false;
}

void TriggerBit::GetTrigCondition(const string& trigger_algorithm) {

  vector<string> ElecSequences;
  vector<string> MuonSequences;
  vector<string> JetSequences;
  vector<string> TauJetSequences;
  vector<string> EtmisSequences;
  vector<string> GammaSequences;

  char *result = NULL;
  result = strtok( (char*) trigger_algorithm.c_str(),"&");
  while( result != NULL )
     {
       if(strstr (result,"ELEC"))ElecSequences.push_back(result);
       if(strstr (result,"MUON"))MuonSequences.push_back(result);
       if(strstr (result,"JET"))JetSequences.push_back(result);
       if(strstr (result,"TAUJET"))TauJetSequences.push_back(result);
       if(strstr (result,"ETMIS"))EtmisSequences.push_back(result);
       if(strstr (result,"GAMMA"))GammaSequences.push_back(result);
       result = strtok( NULL,"&");
     }

  ElecValues = GetCuts(ElecSequences);
  MuonValues = GetCuts(MuonSequences);
  JetValues = GetCuts(JetSequences);
  TauJetValues = GetCuts(TauJetSequences);
  EtmisValues = GetCuts(EtmisSequences);
  GammaValues = GetCuts(GammaSequences);

}
void TriggerBit::PrintTrigCondition(string LogName,int i) 
{
  int elec_size = TriggerBit::ElecValues.size();
  int muon_size = TriggerBit::MuonValues.size();
  int jet_size = TriggerBit::JetValues.size();
  int taujet_size = TriggerBit::TauJetValues.size();
  int gamma_size = TriggerBit::GammaValues.size();
  int etmis_size = TriggerBit::EtmisValues.size();
  
  ofstream f_out(LogName.c_str(),ios::app);

  f_out <<"*                                                                    *"<<"\n";
  f_out << left << setw(25) <<"# Trigger bit number "<<""
        << left << setw(35) <<i       <<""<< right << setw(10)<<"*"<<"\n";
  f_out <<"#*************************"<<"\n";
  f_out << left << setw(45) <<"*   # Number of reconstructed objects required:"<<""<<right << setw(23)<<"*"<<"\n";
  f_out << left << setw(1) <<"*"
        << internal << setw(10) << "Electron"<< ""<< internal << setw(10) << "Muon"    << ""
        << internal << setw(10) << "Jets"    << ""<< internal << setw(10) << "Tau-jets "<< ""
        << internal << setw(10) << "Photons" << ""<< internal << setw(10) << "ETmis "<< ""<< right    << setw(9)  <<"*"<<"\n";
  f_out << left << setw(1) <<"*"
        << internal << setw(10) << elec_size << ""<< internal << setw(10) << muon_size << ""
        << internal << setw(10) << jet_size  << ""<< internal << setw(10) << taujet_size << ""
        << internal << setw(10) << gamma_size<< ""<< internal << setw(10) << etmis_size << ""<< right << setw(9)<<"*"<<"\n";
  f_out << left << setw(45) <<"*   # Values of the related thresholds: "<<""<<right << setw(25)<<"*"<<"\n";
  if(elec_size!=0){
  f_out << left << setw(20) <<"*    - Electrons: "<<"";
  for(int i=0;i<elec_size;i++){f_out << left << setw(5) << TriggerBit::ElecValues[i]<<"";}f_out <<"\n";}

  if(muon_size!=0){
  f_out << left << setw(20) <<"*    - Muons: "<<"";
  for(int i=0;i<muon_size;i++){f_out << left << setw(5) << TriggerBit::MuonValues[i]<<"";}f_out <<"\n";}

  if(jet_size!=0){
  f_out << left << setw(20) <<"*    - Jets: "<<"";
  for(int i=0;i<jet_size;i++){f_out << left << setw(5) << TriggerBit::JetValues[i]<<"";}f_out <<"\n";}

  if(taujet_size!=0){
  f_out << left << setw(20) <<"*    - Tau-jets: "<<"";
  for(int i=0;i<taujet_size;i++){f_out << left << setw(5) << TriggerBit::TauJetValues[i]<<"";}f_out <<"\n";}
 
  if(gamma_size!=0){
  f_out << left << setw(20) <<"*    - Photons: "<<"";
  for(int i=0;i<gamma_size;i++){f_out << left << setw(5) << TriggerBit::GammaValues[i]<<"";}f_out <<"\n";}
 
  if(etmis_size!=0){
  f_out << left << setw(20) <<"*    - ETMIS: "<<"";
  for(int i=0;i<etmis_size;i++){f_out << left << setw(5) << TriggerBit::EtmisValues[i]<<"";}f_out <<"\n";}
  f_out <<"*                                                                    *"<<"\n";



}

vector<float> TriggerBit::GetCuts(const vector<string> &Sequences)
{
  vector<float> OrderedValue;
  string ptVal;
  Int_t PtVal=0;
  for(unsigned int i=0; i < Sequences.size(); i++) {
    ptVal = Sequences[i].substr( Sequences[i].find("'")+1) ;
    PtVal = atoi( ptVal.c_str() );
    OrderedValue.push_back(PtVal);
   }
  return OrderedValue;

}


